@@ -2,7 +2,12 @@
 # All rights reserved.
 
 from hypernetx import Hypergraph
-from .util import get_frozenset_label, get_set_layering
+from .util import (
+    get_frozenset_label,
+    get_set_layering,
+    inflate_kwargs,
+    transpose_inflated_kwargs,
+)
 
 import matplotlib.pyplot as plt
 from matplotlib.collections import PolyCollection, LineCollection, CircleCollection
@@ -18,21 +23,21 @@ from scipy.spatial import ConvexHull
 from scipy.spatial import Voronoi
 
 # increases the default figure size to 8in square.
-plt.rcParams['figure.figsize'] = (8, 8)
+plt.rcParams["figure.figsize"] = (8, 8)
 
 N_CONTROL_POINTS = 24
 
-theta = np.linspace(0, 2*np.pi, N_CONTROL_POINTS + 1)[:-1]
+theta = np.linspace(0, 2 * np.pi, N_CONTROL_POINTS + 1)[:-1]
+
+cp = np.vstack((np.cos(theta), np.sin(theta))).T
 
-cp = np.vstack((np.cos(theta),
-                np.sin(theta))).T
 
 def layout_node_link(H, layout=nx.spring_layout, **kwargs):
-    '''
+    """
     Helper function to use a NetwrokX-like graph layout algorithm on a Hypergraph
 
     The hypergraph is converted to a bipartite graph, allowing the usual graph layout
-    techniques to be applied. 
+    techniques to be applied.
 
     Parameters
     ----------
@@ -46,35 +51,13 @@ def layout_node_link(H, layout=nx.spring_layout, **kwargs):
     Returns
     -------
     dict
-        mapping of node and edge positions to R^2        
-    '''
+        mapping of node and edge positions to R^2
+    """
     return layout(H.bipartite(), **kwargs)
 
-def inflate_kwargs(n, **kwargs):
-    '''
-    Helper function to expand keyword arguments.
-
-    Parameters
-    ----------
-    n: int
-        length of resulting list if argument is expanded
-    kwargs: dict
-        keyword arguments to be expanded
-
-    Returns
-    -------
-    dict
-        dictionary with same keys as kwargs and whose values are lists of length n
-    '''
-
-    inflated = {k: v if type(v) not in {str, tuple, int, float} else [v]*n
-            for k, v in kwargs.items()}
-
-    return [dict(zip(inflated, v))
-            for v in zip(*inflated.values())]
 
 def get_default_radius(H, pos):
-    '''
+    """
     Calculate a reasonable default node radius
 
     This function iterates over the hyper edges and finds the most distant
@@ -93,16 +76,16 @@ def get_default_radius(H, pos):
     float
         the recommended radius
 
-    '''
+    """
     if len(H) > 1:
-        return .0125*np.median([
-            pdist(np.vstack(list(map(pos.get, H.nodes)))).max()
-            for nodes in H.edges()
-        ])
+        return 0.0125 * np.median(
+            [pdist(np.vstack(list(map(pos.get, H.nodes)))).max() for nodes in H.edges()]
+        )
     return 1
 
+
 def draw_hyper_edge_labels(H, polys, labels={}, ax=None, **kwargs):
-    '''
+    """
     Draws a label on the hyper edge boundary.
 
     Should be passed Matplotlib PolyCollection representing the hyper-edges, see
@@ -114,7 +97,7 @@ def draw_hyper_edge_labels(H, polys, labels={}, ax=None, **kwargs):
     Parameters
     ----------
     H: Hypergraph
-        the entity to be drawn 
+        the entity to be drawn
     polys: PolyCollection
         collection of polygons returned by draw_hyper_edges
     labels: dict
@@ -123,36 +106,36 @@ def draw_hyper_edge_labels(H, polys, labels={}, ax=None, **kwargs):
         matplotlib axis on which the plot is rendered
     kwargs: dict
         Keyword arguments are passed through to Matplotlib's annotate function.
-    
-    '''
+
+    """
     ax = ax or plt.gca()
 
-    params = inflate_kwargs(len(H.edges), **kwargs)
+    params = transpose_inflated_kwargs(inflate_kwargs(H.edges, kwargs))
 
     for edge, path, params in zip(H.edges, polys.get_paths(), params):
         s = labels.get(edge, edge)
 
         # calculate the xy location of the annotation
         # this is the midpoint of the pair of adjacent points the most distant
-        d = ((path.vertices[:-1] - path.vertices[1:])**2).sum(axis=1)
+        d = ((path.vertices[:-1] - path.vertices[1:]) ** 2).sum(axis=1)
         i = d.argmax()
 
-        x1, x2 = path.vertices[i:i+2]
-        x, y = (x2 - x1)
-        theta = 360*np.arctan2(y, x)/(2*np.pi)
-        theta = (theta + 360)%360
+        x1, x2 = path.vertices[i : i + 2]
+        x, y = x2 - x1
+        theta = 360 * np.arctan2(y, x) / (2 * np.pi)
+        theta = (theta + 360) % 360
 
         while theta > 90:
             theta -= 180
 
         # the string is a comma separated list of the edge uid
-        ax.annotate(s, (x1 + x2)/2,
-                    rotation=theta,
-                    ha='center', va='center',
-                    **params)
+        ax.annotate(
+            s, (x1 + x2) / 2, rotation=theta, ha="center", va="center", **params
+        )
+
 
 def layout_hyper_edges(H, pos, node_radius={}, dr=None):
-    '''
+    """
     Draws a convex hull for each edge in H.
 
     Position of the nodes in the graph is specified by the position dictionary,
@@ -163,7 +146,7 @@ def layout_hyper_edges(H, pos, node_radius={}, dr=None):
     Parameters
     ----------
     H: Hypergraph
-        the entity to be drawn 
+        the entity to be drawn
     pos: dict
         mapping of node and edge positions to R^2
     node_radius: dict
@@ -177,7 +160,7 @@ def layout_hyper_edges(H, pos, node_radius={}, dr=None):
     -------
     dict
         A mapping from hyper edge ids to paths (Nx2 numpy matrices)
-    '''
+    """
 
     if len(node_radius):
         r0 = min(node_radius.values())
@@ -185,36 +168,42 @@ def layout_hyper_edges(H, pos, node_radius={}, dr=None):
         r0 = get_default_radius(H, pos)
 
     dr = dr or r0
-    
+
     levels = get_set_layering(H)
 
-    radii = {v: {v: i for i, v in enumerate(sorted(e, key=levels.get))}
-             for v, e in H.dual().edges.elements.items()}
+    radii = {
+        v: {v: i for i, v in enumerate(sorted(e, key=levels.get))}
+        for v, e in H.dual().edges.elements.items()
+    }
 
-    def get_padded_hull(edge):
+    def get_padded_hull(uid, edge):
         # make sure the edge contains at least one node
         if len(edge):
-            points = np.vstack([cp*(node_radius.get(v, r0) + dr*(2 + radii[v][edge.uid])) + pos[v]
-                                for v in edge])
-        # if not, draw an empty edge centered aroudn the location of the edge node (in the bipartite graph)
+            points = np.vstack(
+                [
+                    cp * (node_radius.get(v, r0) + dr * (2 + radii[v][uid])) + pos[v]
+                    for v in edge
+                ]
+            )
+        # if not, draw an empty edge centered around the location of the edge node (in the bipartite graph)
         else:
-            points = 4*r0*cp + pos[edge.uid]
-        
+            points = 4 * r0 * cp + pos[uid]
+
         hull = ConvexHull(points)
-        
+
         return hull.points[hull.vertices]
 
-    return [get_padded_hull(e)
-            for e in H.edges()]
+    return [get_padded_hull(uid, list(H.edges[uid])) for uid in H.edges]
+
 
 def draw_hyper_edges(H, pos, ax=None, node_radius={}, dr=None, **kwargs):
-    '''
+    """
     Draws a convex hull around the nodes contained within each edge in H
 
     Parameters
     ----------
     H: Hypergraph
-        the entity to be drawn 
+        the entity to be drawn
     pos: dict
         mapping of node and edge positions to R^2
     node_radius: dict
@@ -230,17 +219,18 @@ def draw_hyper_edges(H, pos, ax=None, node_radius={}, dr=None, **kwargs):
     -------
     PolyCollection
         a Matplotlib PolyCollection that can be further styled
-    '''
+    """
     points = layout_hyper_edges(H, pos, node_radius=node_radius, dr=dr)
 
-    polys = PolyCollection(points, **kwargs)
+    polys = PolyCollection(points, **inflate_kwargs(H.edges, kwargs))
 
     (ax or plt.gca()).add_collection(polys)
-    
+
     return polys
 
+
 def draw_hyper_nodes(H, pos, node_radius={}, r0=None, ax=None, **kwargs):
-    '''
+    """
     Draws a circle for each node in H.
 
     The position of each node is specified by the a dictionary/list-like, pos,
@@ -253,7 +243,7 @@ def draw_hyper_nodes(H, pos, node_radius={}, r0=None, ax=None, **kwargs):
     Parameters
     ----------
     H: Hypergraph
-        the entity to be drawn 
+        the entity to be drawn
     pos: dict
         mapping of node and edge positions to R^2
     node_radius: dict
@@ -269,24 +259,25 @@ def draw_hyper_nodes(H, pos, node_radius={}, r0=None, ax=None, **kwargs):
     -------
     PolyCollection
         a Matplotlib PolyCollection that can be further styled
-    '''
+    """
 
     ax = ax or plt.gca()
 
     r0 = r0 or get_default_radius(H, pos)
 
-    points = [node_radius.get(v, r0)*cp + pos[v] for v in H.nodes]
+    points = [node_radius.get(v, r0) * cp + pos[v] for v in H.nodes]
 
-    kwargs.setdefault('facecolors', 'black')
+    kwargs.setdefault("facecolors", "black")
 
-    circles = PolyCollection(points, **kwargs)
+    circles = PolyCollection(points, **inflate_kwargs(H, kwargs))
 
     ax.add_collection(circles)
 
     return circles
 
+
 def draw_hyper_labels(H, pos, node_radius={}, ax=None, labels={}, **kwargs):
-    '''
+    """
     Draws text labels for the hypergraph nodes.
 
     The label is drawn to the right of the node. The node radius is needed (see
@@ -302,7 +293,7 @@ def draw_hyper_labels(H, pos, node_radius={}, ax=None, labels={}, **kwargs):
     Parameters
     ----------
     H: Hypergraph
-        the entity to be drawn 
+        the entity to be drawn
     pos: dict
         mapping of node and edge positions to R^2
     node_radius: dict
@@ -314,40 +305,54 @@ def draw_hyper_labels(H, pos, node_radius={}, ax=None, labels={}, **kwargs):
     kwargs: dict
         keyword arguments passed to matplotlib.annotate
 
-    '''
+    """
     ax = ax or plt.gca()
 
-    for v in H.nodes:
-        xy = np.array([node_radius.get(v, 0), 0]) + pos[v]
-        ax.annotate(labels.get(v, v), xy,
-                    **{k: (d[v] if hasattr(d, '__getitem__') and type(d) not in {str, tuple} else d)
-                       for k,d in kwargs.items()})
+    params = transpose_inflated_kwargs(inflate_kwargs(H.nodes, kwargs))
 
-def draw(H,
-         pos=None,
-         with_color=True,
-         with_node_counts=False,
-         with_edge_counts=False,
-         layout=nx.spring_layout,
-         layout_kwargs={},
-         ax=None,
-         node_radius=None,
-         edges_kwargs={},
-         nodes_kwargs={},
-         edge_labels={},
-         edge_labels_kwargs={},
-         node_labels={},
-         node_labels_kwargs={},
-         with_edge_labels=True,
-         with_node_labels=True,
-         label_alpha=.35
-        ):
-    '''
+    for v, v_kwargs in zip(H.nodes, params):
+        xy = np.array([node_radius.get(v, 0), 0]) + pos[v]
+        ax.annotate(
+            labels.get(v, v),
+            xy,
+            **{
+                k: (
+                    d[v]
+                    if hasattr(d, "__getitem__") and type(d) not in {str, tuple}
+                    else d
+                )
+                for k, d in kwargs.items()
+            }
+        )
+
+
+def draw(
+    H,
+    pos=None,
+    with_color=True,
+    with_node_counts=False,
+    with_edge_counts=False,
+    layout=nx.spring_layout,
+    layout_kwargs={},
+    ax=None,
+    node_radius=None,
+    edges_kwargs={},
+    nodes_kwargs={},
+    edge_labels={},
+    edge_labels_kwargs={},
+    node_labels={},
+    node_labels_kwargs={},
+    with_edge_labels=True,
+    with_node_labels=True,
+    label_alpha=0.35,
+    return_pos=False,
+):
+    """
     Draw a hypergraph as a Matplotlib figure
 
     By default this will draw a colorful "rubber band" like hypergraph, where
     convex hulls represent edges and are drawn around the nodes they contain.
-    
+
     This is a convenience function that wraps calls with sensible parameters to
     the following lower-level drawing functions:
 
@@ -381,12 +386,12 @@ def draw(H,
     implies that the sets intersect, but sometimes sets overlap if there is no
     intersection. It is not possible, in general, to draw a "correct" hypergraph
     this way for an arbitrary hypergraph, in the same way that not all graphs
-    have planar drawings.    
+    have planar drawings.
 
     Parameters
     ----------
     H: Hypergraph
-        the entity to be drawn 
+        the entity to be drawn
     pos: dict
         mapping of node and edge positions to R^2
     with_color: bool
@@ -408,7 +413,7 @@ def draw(H,
     nodes_kwargs: dict
         keyword arguments passed to matplotlib.collections.PolyCollection for nodes
     edge_labels_kwargs: dict
-        keyword arguments passed to matplotlib.annotate for edge laberls
+        keyword arguments passed to matplotlib.annotate for edge labels
     node_labels_kwargs: dict
         keyword argumetns passed to matplotlib.annotate for node labels
     with_edge_labels: bool
@@ -417,79 +422,68 @@ def draw(H,
         set to False to make node labels invisible
     label_alpha: float
         the transparency (alpha) of the box behind text drawn in the figure
-    '''
+    """
 
     ax = ax or plt.gca()
-    
+
     if pos is None:
         pos = layout_node_link(H, layout=layout, **layout_kwargs)
 
     r0 = get_default_radius(H, pos)
-    a0 = np.pi*r0**2
+    a0 = np.pi * r0 ** 2
 
     def get_node_radius(v):
         if node_radius is None:
-            return np.sqrt(a0*(len(v) if type(v) == frozenset else 1)/np.pi)
-        elif hasattr(node_radius, 'get'):
-            return node_radius.get(v, 1)*r0
-        return node_radius*r0
+            return np.sqrt(a0 * (len(v) if type(v) == frozenset else 1) / np.pi)
+        elif hasattr(node_radius, "get"):
+            return node_radius.get(v, 1) * r0
+        return node_radius * r0
 
     # guarantee that node radius is a dictionary mapping nodes to values
-    node_radius = {
-        v: get_node_radius(v)
-        for v in H.nodes
-    }
+    node_radius = {v: get_node_radius(v) for v in H.nodes}
 
     # for convenience, we are using setdefault to mutate the argument
     # however, we need to copy this to prevent side-effects
     edges_kwargs = edges_kwargs.copy()
-    edges_kwargs.setdefault('edgecolors', plt.cm.tab10(np.arange(len(H.edges))%10))
-    edges_kwargs.setdefault('facecolors', 'none')
+    edges_kwargs.setdefault("edgecolors", plt.cm.tab10(np.arange(len(H.edges)) % 10))
+    edges_kwargs.setdefault("facecolors", "none")
 
-    polys = draw_hyper_edges(H, pos,
-                             node_radius=node_radius,
-                             ax=ax,
-                             **edges_kwargs
-                            )
+    polys = draw_hyper_edges(H, pos, node_radius=node_radius, ax=ax, **edges_kwargs)
 
     if with_edge_labels:
         labels = get_frozenset_label(
-            H.edges,
-            count=with_edge_counts,
-            override=edge_labels
+            H.edges, count=with_edge_counts, override=edge_labels
         )
 
-        draw_hyper_edge_labels(H, polys,
-                               color=edges_kwargs['edgecolors'],
-                               backgroundcolor=(1, 1, 1, label_alpha),
-                               labels=labels,
-                               ax=ax,
-                               **edge_labels_kwargs
-                               )
+        draw_hyper_edge_labels(
+            H,
+            polys,
+            color=edges_kwargs["edgecolors"],
+            backgroundcolor=(1, 1, 1, label_alpha),
+            labels=labels,
+            ax=ax,
+            **edge_labels_kwargs
+        )
 
     if with_node_labels:
         labels = get_frozenset_label(
-            H.nodes,
-            count=with_node_counts,
-            override=node_labels
+            H.nodes, count=with_node_counts, override=node_labels
         )
 
-        draw_hyper_labels(H, pos,
-                          node_radius=node_radius,
-                          labels=labels,
-                          ax=ax,
-                          va='center',
-                          xytext=(5, 0),
-                          textcoords='offset points',
-                          backgroundcolor=(1, 1, 1, label_alpha),
-                          **node_labels_kwargs
-                          )
+        draw_hyper_labels(
+            H,
+            pos,
+            node_radius=node_radius,
+            labels=labels,
+            ax=ax,
+            va="center",
+            xytext=(5, 0),
+            textcoords="offset points",
+            backgroundcolor=(1, 1, 1, label_alpha),
+            **node_labels_kwargs
+        )
 
-    draw_hyper_nodes(H, pos,
-                     node_radius=node_radius,
-                     ax=ax,
-                     **nodes_kwargs
-                    )
+    draw_hyper_nodes(H, pos, node_radius=node_radius, ax=ax, **nodes_kwargs)
 
     if len(H.nodes) == 1:
         x, y = pos[list(H.nodes)[0]]
@@ -497,7 +491,8 @@ def draw(H,
 
         ax.axis([x - s, x + s, y - s, y + s])
     else:
-        ax.axis('equal')
+        ax.axis("equal")
 
-    ax.axis('off')
-    
+    ax.axis("off")
+    if return_pos:
+        return pos
